/*
 * Daneel - Dalvik to Java bytecode compiler
 * Copyright (C) 2011  IcedRobot team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.icedrobot.daneel.dex;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.nio.ByteBuffer;
import java.util.Arrays;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class HeaderTest {

    private static final byte[] TEST_HEADER = new byte[] { 0x64, 0x65, 0x78,
            0x0a, 0x30, 0x33, 0x35,
            0x00, // Magic 0-7
            0x78, 0x56, 0x34,
            0x12, // Checksum 8-11
            0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
            19, // Signature 12-31
            0x78, 0x56, 0x34, 0x12, // File size 32-35
            0x78, 0x56, 0x34, 0x12, // Header length 36-39
            0x78, 0x56, 0x34, 0x12, // Endian tag 40-43
            0x78, 0x56, 0x34, 0x12, // Link size 44-47
            0x12, 0x34, 0x56, 0x78, // Link off 48-51
            0x12, 0x34, 0x56, 0x78, // Map off 52-55
            0x78, 0x56, 0x34, 0x12, // String ids size 56-59
            0x12, 0x34, 0x56, 0x78, // String ids off 60-63
            0x78, 0x56, 0x34, 0x12, // Type ids size 64-67
            0x12, 0x34, 0x56, 0x78, // Type ids off 68-71
            0x78, 0x56, 0x34, 0x12, // Proto ids size 72-75
            0x12, 0x34, 0x56, 0x78, // Proto ids off 76-79
            0x78, 0x56, 0x34, 0x12, // Field ids size 80-83
            0x12, 0x34, 0x56, 0x78, // Field ids off 84-87
            0x78, 0x56, 0x34, 0x12, // Method ids size 88-91
            0x12, 0x34, 0x56, 0x78, // Method ids off 92-95
            0x78, 0x56, 0x34, 0x12, // Class defs size 96-99
            0x12, 0x34, 0x56, 0x78, // Class defs off 100-103
            0x78, 0x56, 0x34, 0x12, // Data size 104-107
            0x12, 0x34, 0x56, 0x78, // Data off 108-111
    };

    private ByteBuffer testHeader;

    @Before
    public void setUp() {
        testHeader = ByteBuffer.wrap(TEST_HEADER);
    }

    @After
    public void tearDown() {
        testHeader = null;
    }

    @Test
    public void testMagic() {
        Header.parse(testHeader);
        // Here we simply expect to not throw any exception.
    }

    @Test
    public void testChecksum() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getChecksum());
    }

    @Test
    public void testSignature() {
        Header header = Header.parse(testHeader);
        byte[] expected = new byte[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
                12, 13, 14, 15, 16, 17, 18, 19 };
        assertArrayEquals(expected, header.getSignature());
    }

    @Test
    public void testFileLength() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getFileLength());
    }

    @Test
    public void testHeaderLength() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getHeaderLength());
    }

    @Test
    public void testHeaderTooShort() {
        try {
            byte[] wrong = Arrays.copyOfRange(TEST_HEADER, 0, 39);
            Header.parse(ByteBuffer.wrap(wrong));
            fail();
        } catch (DexParseException ex) {
            // Expected.
        }
    }

    @Test
    public void testEndianTag() {
        Header header = Header.parse(testHeader);
        assertEquals(Header.Endianess.ENDIAN_CONSTANT, header.getEndianTag());
    }

    @Test
    public void testEndianTagBigEndian() {
        byte[] bigEndianHeader = Arrays.copyOf(TEST_HEADER, TEST_HEADER.length);
        System.arraycopy(new byte[] { 0x12, 0x34, 0x56, 0x78 }, 0,
                bigEndianHeader, 40, 4);
        Header header = Header.parse(ByteBuffer.wrap(bigEndianHeader));
        assertEquals(Header.Endianess.REVERSE_ENDIAN_CONSTANT,
                header.getEndianTag());
    }

    @Test
    public void testEndianTagWrongEndian() {
        try {
            byte[] wrongEndianHeader = Arrays.copyOf(TEST_HEADER,
                    TEST_HEADER.length);
            System.arraycopy(new byte[] { 0x56, 0x78, 0x12, 0x34 }, 0,
                    wrongEndianHeader, 40, 4);
            Header.parse(ByteBuffer.wrap(wrongEndianHeader));
        } catch (DexParseException ex) {
            // Expected.
        }
    }

    @Test
    public void testLinkSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getLinkSize());
    }

    @Test
    public void testLinkOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getLinkOff());
    }

    @Test
    public void testMapOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getMapOff());
    }

    @Test
    public void testStringIdsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getStringIdsSize());
    }

    @Test
    public void testStringIdsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getStringIdsOff());
    }

    @Test
    public void testTypeIdsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getTypeIdsSize());
    }

    @Test
    public void testTypeIdsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getTypeIdsOff());
    }

    @Test
    public void testProtoIdsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getProtoIdsSize());
    }

    @Test
    public void testProtoIdsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getProtoIdsOff());
    }

    @Test
    public void testFieldIdsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getFieldIdsSize());
    }

    @Test
    public void testFieldIdsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getFieldIdsOff());
    }

    @Test
    public void testMethodIdsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getMethodIdsSize());
    }

    @Test
    public void testMethodIdsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getMethodIdsOff());
    }

    @Test
    public void testClassDefsSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getClassDefsSize());
    }

    @Test
    public void testClassDefsOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getClassDefsOff());
    }

    @Test
    public void testDataSize() {
        Header header = Header.parse(testHeader);
        assertEquals(0x12345678, header.getDataSize());
    }

    @Test
    public void testDataOff() {
        Header header = Header.parse(testHeader);
        assertEquals(0x78563412, header.getDataOff());
    }
}
